# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel
from .dispute_stage import DisputeStage
from .dispute_status import DisputeStatus

__all__ = ["DisputeListResponse"]


class DisputeListResponse(BaseModel):
    amount: str
    """
    The amount involved in the dispute, represented as a string to accommodate
    precision.
    """

    business_id: str
    """The unique identifier of the business involved in the dispute."""

    created_at: datetime
    """The timestamp of when the dispute was created, in UTC."""

    currency: str
    """The currency of the disputed amount, represented as an ISO 4217 currency code."""

    dispute_id: str
    """The unique identifier of the dispute."""

    dispute_stage: DisputeStage
    """The current stage of the dispute process."""

    dispute_status: DisputeStatus
    """The current status of the dispute."""

    payment_id: str
    """The unique identifier of the payment associated with the dispute."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import Currency, payment_list_params, payment_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.payment import Payment
from ..types.currency import Currency
from ..types.payment_method_types import PaymentMethodTypes
from ..types.billing_address_param import BillingAddressParam
from ..types.payment_list_response import PaymentListResponse
from ..types.customer_request_param import CustomerRequestParam
from ..types.payment_create_response import PaymentCreateResponse
from ..types.one_time_product_cart_item_param import OneTimeProductCartItemParam
from ..types.payment_retrieve_line_items_response import PaymentRetrieveLineItemsResponse

__all__ = ["PaymentsResource", "AsyncPaymentsResource"]


class PaymentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return PaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return PaymentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        billing: BillingAddressParam,
        customer: CustomerRequestParam,
        product_cart: Iterable[OneTimeProductCartItemParam],
        allowed_payment_method_types: Optional[List[PaymentMethodTypes]] | Omit = omit,
        billing_currency: Optional[Currency] | Omit = omit,
        discount_code: Optional[str] | Omit = omit,
        force_3ds: Optional[bool] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        payment_link: Optional[bool] | Omit = omit,
        return_url: Optional[str] | Omit = omit,
        show_saved_payment_methods: bool | Omit = omit,
        tax_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentCreateResponse:
        """
        Args:
          billing: Billing address details for the payment

          customer: Customer information for the payment

          product_cart: List of products in the cart. Must contain at least 1 and at most 100 items.

          allowed_payment_method_types: List of payment methods allowed during checkout.

              Customers will **never** see payment methods that are **not** in this list.
              However, adding a method here **does not guarantee** customers will see it.
              Availability still depends on other factors (e.g., customer location, merchant
              settings).

          billing_currency: Fix the currency in which the end customer is billed. If Dodo Payments cannot
              support that currency for this transaction, it will not proceed

          discount_code: Discount Code to apply to the transaction

          force_3ds: Override merchant default 3DS behaviour for this payment

          metadata: Additional metadata associated with the payment. Defaults to empty if not
              provided.

          payment_link: Whether to generate a payment link. Defaults to false if not specified.

          return_url: Optional URL to redirect the customer after payment. Must be a valid URL if
              provided.

          show_saved_payment_methods: Display saved payment methods of a returning customer False by default

          tax_id: Tax ID in case the payment is B2B. If tax id validation fails the payment
              creation will fail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/payments",
            body=maybe_transform(
                {
                    "billing": billing,
                    "customer": customer,
                    "product_cart": product_cart,
                    "allowed_payment_method_types": allowed_payment_method_types,
                    "billing_currency": billing_currency,
                    "discount_code": discount_code,
                    "force_3ds": force_3ds,
                    "metadata": metadata,
                    "payment_link": payment_link,
                    "return_url": return_url,
                    "show_saved_payment_methods": show_saved_payment_methods,
                    "tax_id": tax_id,
                },
                payment_create_params.PaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentCreateResponse,
        )

    def retrieve(
        self,
        payment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Payment:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_id:
            raise ValueError(f"Expected a non-empty value for `payment_id` but received {payment_id!r}")
        return self._get(
            f"/payments/{payment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    def list(
        self,
        *,
        brand_id: str | Omit = omit,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        customer_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: Literal[
            "succeeded",
            "failed",
            "cancelled",
            "processing",
            "requires_customer_action",
            "requires_merchant_action",
            "requires_payment_method",
            "requires_confirmation",
            "requires_capture",
            "partially_captured",
            "partially_captured_and_capturable",
        ]
        | Omit = omit,
        subscription_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[PaymentListResponse]:
        """
        Args:
          brand_id: filter by Brand id

          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          customer_id: Filter by customer id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          status: Filter by status

          subscription_id: Filter by subscription id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/payments",
            page=SyncDefaultPageNumberPagination[PaymentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                        "subscription_id": subscription_id,
                    },
                    payment_list_params.PaymentListParams,
                ),
            ),
            model=PaymentListResponse,
        )

    def retrieve_line_items(
        self,
        payment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentRetrieveLineItemsResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_id:
            raise ValueError(f"Expected a non-empty value for `payment_id` but received {payment_id!r}")
        return self._get(
            f"/payments/{payment_id}/line-items",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentRetrieveLineItemsResponse,
        )


class AsyncPaymentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPaymentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPaymentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPaymentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncPaymentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        billing: BillingAddressParam,
        customer: CustomerRequestParam,
        product_cart: Iterable[OneTimeProductCartItemParam],
        allowed_payment_method_types: Optional[List[PaymentMethodTypes]] | Omit = omit,
        billing_currency: Optional[Currency] | Omit = omit,
        discount_code: Optional[str] | Omit = omit,
        force_3ds: Optional[bool] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        payment_link: Optional[bool] | Omit = omit,
        return_url: Optional[str] | Omit = omit,
        show_saved_payment_methods: bool | Omit = omit,
        tax_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentCreateResponse:
        """
        Args:
          billing: Billing address details for the payment

          customer: Customer information for the payment

          product_cart: List of products in the cart. Must contain at least 1 and at most 100 items.

          allowed_payment_method_types: List of payment methods allowed during checkout.

              Customers will **never** see payment methods that are **not** in this list.
              However, adding a method here **does not guarantee** customers will see it.
              Availability still depends on other factors (e.g., customer location, merchant
              settings).

          billing_currency: Fix the currency in which the end customer is billed. If Dodo Payments cannot
              support that currency for this transaction, it will not proceed

          discount_code: Discount Code to apply to the transaction

          force_3ds: Override merchant default 3DS behaviour for this payment

          metadata: Additional metadata associated with the payment. Defaults to empty if not
              provided.

          payment_link: Whether to generate a payment link. Defaults to false if not specified.

          return_url: Optional URL to redirect the customer after payment. Must be a valid URL if
              provided.

          show_saved_payment_methods: Display saved payment methods of a returning customer False by default

          tax_id: Tax ID in case the payment is B2B. If tax id validation fails the payment
              creation will fail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/payments",
            body=await async_maybe_transform(
                {
                    "billing": billing,
                    "customer": customer,
                    "product_cart": product_cart,
                    "allowed_payment_method_types": allowed_payment_method_types,
                    "billing_currency": billing_currency,
                    "discount_code": discount_code,
                    "force_3ds": force_3ds,
                    "metadata": metadata,
                    "payment_link": payment_link,
                    "return_url": return_url,
                    "show_saved_payment_methods": show_saved_payment_methods,
                    "tax_id": tax_id,
                },
                payment_create_params.PaymentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentCreateResponse,
        )

    async def retrieve(
        self,
        payment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Payment:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_id:
            raise ValueError(f"Expected a non-empty value for `payment_id` but received {payment_id!r}")
        return await self._get(
            f"/payments/{payment_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Payment,
        )

    def list(
        self,
        *,
        brand_id: str | Omit = omit,
        created_at_gte: Union[str, datetime] | Omit = omit,
        created_at_lte: Union[str, datetime] | Omit = omit,
        customer_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        status: Literal[
            "succeeded",
            "failed",
            "cancelled",
            "processing",
            "requires_customer_action",
            "requires_merchant_action",
            "requires_payment_method",
            "requires_confirmation",
            "requires_capture",
            "partially_captured",
            "partially_captured_and_capturable",
        ]
        | Omit = omit,
        subscription_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PaymentListResponse, AsyncDefaultPageNumberPagination[PaymentListResponse]]:
        """
        Args:
          brand_id: filter by Brand id

          created_at_gte: Get events after this created time

          created_at_lte: Get events created before this time

          customer_id: Filter by customer id

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          status: Filter by status

          subscription_id: Filter by subscription id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/payments",
            page=AsyncDefaultPageNumberPagination[PaymentListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "created_at_gte": created_at_gte,
                        "created_at_lte": created_at_lte,
                        "customer_id": customer_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "status": status,
                        "subscription_id": subscription_id,
                    },
                    payment_list_params.PaymentListParams,
                ),
            ),
            model=PaymentListResponse,
        )

    async def retrieve_line_items(
        self,
        payment_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PaymentRetrieveLineItemsResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not payment_id:
            raise ValueError(f"Expected a non-empty value for `payment_id` but received {payment_id!r}")
        return await self._get(
            f"/payments/{payment_id}/line-items",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PaymentRetrieveLineItemsResponse,
        )


class PaymentsResourceWithRawResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.create = to_raw_response_wrapper(
            payments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            payments.retrieve,
        )
        self.list = to_raw_response_wrapper(
            payments.list,
        )
        self.retrieve_line_items = to_raw_response_wrapper(
            payments.retrieve_line_items,
        )


class AsyncPaymentsResourceWithRawResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.create = async_to_raw_response_wrapper(
            payments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            payments.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            payments.list,
        )
        self.retrieve_line_items = async_to_raw_response_wrapper(
            payments.retrieve_line_items,
        )


class PaymentsResourceWithStreamingResponse:
    def __init__(self, payments: PaymentsResource) -> None:
        self._payments = payments

        self.create = to_streamed_response_wrapper(
            payments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            payments.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            payments.list,
        )
        self.retrieve_line_items = to_streamed_response_wrapper(
            payments.retrieve_line_items,
        )


class AsyncPaymentsResourceWithStreamingResponse:
    def __init__(self, payments: AsyncPaymentsResource) -> None:
        self._payments = payments

        self.create = async_to_streamed_response_wrapper(
            payments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            payments.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            payments.list,
        )
        self.retrieve_line_items = async_to_streamed_response_wrapper(
            payments.retrieve_line_items,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .customer_limited_details import CustomerLimitedDetails

__all__ = ["LicenseActivateResponse", "Product"]


class Product(BaseModel):
    product_id: str
    """Unique identifier for the product."""

    name: Optional[str] = None
    """Name of the product, if set by the merchant."""


class LicenseActivateResponse(BaseModel):
    id: str
    """License key instance ID"""

    business_id: str
    """Business ID"""

    created_at: datetime
    """Creation timestamp"""

    customer: CustomerLimitedDetails
    """Limited customer details associated with the license key."""

    license_key_id: str
    """Associated license key ID"""

    name: str
    """Instance name"""

    product: Product
    """Related product info. Present if the license key is tied to a product."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ..types import usage_event_list_params, usage_event_ingest_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from ..types.event import Event
from .._base_client import AsyncPaginator, make_request_options
from ..types.event_input_param import EventInputParam
from ..types.usage_event_ingest_response import UsageEventIngestResponse

__all__ = ["UsageEventsResource", "AsyncUsageEventsResource"]


class UsageEventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return UsageEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return UsageEventsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """Fetch detailed information about a single event using its unique event ID.

        This
        endpoint is useful for:

        - Debugging specific event ingestion issues
        - Retrieving event details for customer support
        - Validating that events were processed correctly
        - Getting the complete metadata for an event

        ## Event ID Format:

        The event ID should be the same value that was provided during event ingestion
        via the `/events/ingest` endpoint. Event IDs are case-sensitive and must match
        exactly.

        ## Response Details:

        The response includes all event data including:

        - Complete metadata key-value pairs
        - Original timestamp (preserved from ingestion)
        - Customer and business association
        - Event name and processing information

        ## Example Usage:

        ```text
        GET /events/api_call_12345
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return self._get(
            f"/events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Event,
        )

    def list(
        self,
        *,
        customer_id: str | Omit = omit,
        end: Union[str, datetime] | Omit = omit,
        event_name: str | Omit = omit,
        meter_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[Event]:
        """Fetch events from your account with powerful filtering capabilities.

        This
        endpoint is ideal for:

        - Debugging event ingestion issues
        - Analyzing customer usage patterns
        - Building custom analytics dashboards
        - Auditing billing-related events

        ## Filtering Options:

        - **Customer filtering**: Filter by specific customer ID
        - **Event name filtering**: Filter by event type/name
        - **Meter-based filtering**: Use a meter ID to apply the meter's event name and
          filter criteria automatically
        - **Time range filtering**: Filter events within a specific date range
        - **Pagination**: Navigate through large result sets

        ## Meter Integration:

        When using `meter_id`, the endpoint automatically applies:

        - The meter's configured `event_name` filter
        - The meter's custom filter criteria (if any)
        - If you also provide `event_name`, it must match the meter's event name

        ## Example Queries:

        - Get all events for a customer: `?customer_id=cus_abc123`
        - Get API request events: `?event_name=api_request`
        - Get events from last 24 hours:
          `?start=2024-01-14T10:30:00Z&end=2024-01-15T10:30:00Z`
        - Get events with meter filtering: `?meter_id=mtr_xyz789`
        - Paginate results: `?page_size=50&page_number=2`

        Args:
          customer_id: Filter events by customer ID

          end: Filter events created before this timestamp

          event_name: Filter events by event name. If both event_name and meter_id are provided, they
              must match the meter's configured event_name

          meter_id: Filter events by meter ID. When provided, only events that match the meter's
              event_name and filter criteria will be returned

          page_number: Page number (0-based, default: 0)

          page_size: Number of events to return per page (default: 10)

          start: Filter events created after this timestamp

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/events",
            page=SyncDefaultPageNumberPagination[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "end": end,
                        "event_name": event_name,
                        "meter_id": meter_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "start": start,
                    },
                    usage_event_list_params.UsageEventListParams,
                ),
            ),
            model=Event,
        )

    def ingest(
        self,
        *,
        events: Iterable[EventInputParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageEventIngestResponse:
        """
        This endpoint allows you to ingest custom events that can be used for:

        - Usage-based billing and metering
        - Analytics and reporting
        - Customer behavior tracking

        ## Important Notes:

        - **Duplicate Prevention**:
          - Duplicate `event_id` values within the same request are rejected (entire
            request fails)
          - Subsequent requests with existing `event_id` values are ignored (idempotent
            behavior)
        - **Rate Limiting**: Maximum 1000 events per request
        - **Time Validation**: Events with timestamps older than 1 hour or more than 5
          minutes in the future will be rejected
        - **Metadata Limits**: Maximum 50 key-value pairs per event, keys max 100 chars,
          values max 500 chars

        ## Example Usage:

        ```json
        {
          "events": [
            {
              "event_id": "api_call_12345",
              "customer_id": "cus_abc123",
              "event_name": "api_request",
              "timestamp": "2024-01-15T10:30:00Z",
              "metadata": {
                "endpoint": "/api/v1/users",
                "method": "GET",
                "tokens_used": "150"
              }
            }
          ]
        }
        ```

        Args:
          events: List of events to be pushed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/events/ingest",
            body=maybe_transform({"events": events}, usage_event_ingest_params.UsageEventIngestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageEventIngestResponse,
        )


class AsyncUsageEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncUsageEventsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        event_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Event:
        """Fetch detailed information about a single event using its unique event ID.

        This
        endpoint is useful for:

        - Debugging specific event ingestion issues
        - Retrieving event details for customer support
        - Validating that events were processed correctly
        - Getting the complete metadata for an event

        ## Event ID Format:

        The event ID should be the same value that was provided during event ingestion
        via the `/events/ingest` endpoint. Event IDs are case-sensitive and must match
        exactly.

        ## Response Details:

        The response includes all event data including:

        - Complete metadata key-value pairs
        - Original timestamp (preserved from ingestion)
        - Customer and business association
        - Event name and processing information

        ## Example Usage:

        ```text
        GET /events/api_call_12345
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not event_id:
            raise ValueError(f"Expected a non-empty value for `event_id` but received {event_id!r}")
        return await self._get(
            f"/events/{event_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Event,
        )

    def list(
        self,
        *,
        customer_id: str | Omit = omit,
        end: Union[str, datetime] | Omit = omit,
        event_name: str | Omit = omit,
        meter_id: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Event, AsyncDefaultPageNumberPagination[Event]]:
        """Fetch events from your account with powerful filtering capabilities.

        This
        endpoint is ideal for:

        - Debugging event ingestion issues
        - Analyzing customer usage patterns
        - Building custom analytics dashboards
        - Auditing billing-related events

        ## Filtering Options:

        - **Customer filtering**: Filter by specific customer ID
        - **Event name filtering**: Filter by event type/name
        - **Meter-based filtering**: Use a meter ID to apply the meter's event name and
          filter criteria automatically
        - **Time range filtering**: Filter events within a specific date range
        - **Pagination**: Navigate through large result sets

        ## Meter Integration:

        When using `meter_id`, the endpoint automatically applies:

        - The meter's configured `event_name` filter
        - The meter's custom filter criteria (if any)
        - If you also provide `event_name`, it must match the meter's event name

        ## Example Queries:

        - Get all events for a customer: `?customer_id=cus_abc123`
        - Get API request events: `?event_name=api_request`
        - Get events from last 24 hours:
          `?start=2024-01-14T10:30:00Z&end=2024-01-15T10:30:00Z`
        - Get events with meter filtering: `?meter_id=mtr_xyz789`
        - Paginate results: `?page_size=50&page_number=2`

        Args:
          customer_id: Filter events by customer ID

          end: Filter events created before this timestamp

          event_name: Filter events by event name. If both event_name and meter_id are provided, they
              must match the meter's configured event_name

          meter_id: Filter events by meter ID. When provided, only events that match the meter's
              event_name and filter criteria will be returned

          page_number: Page number (0-based, default: 0)

          page_size: Number of events to return per page (default: 10)

          start: Filter events created after this timestamp

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/events",
            page=AsyncDefaultPageNumberPagination[Event],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "customer_id": customer_id,
                        "end": end,
                        "event_name": event_name,
                        "meter_id": meter_id,
                        "page_number": page_number,
                        "page_size": page_size,
                        "start": start,
                    },
                    usage_event_list_params.UsageEventListParams,
                ),
            ),
            model=Event,
        )

    async def ingest(
        self,
        *,
        events: Iterable[EventInputParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageEventIngestResponse:
        """
        This endpoint allows you to ingest custom events that can be used for:

        - Usage-based billing and metering
        - Analytics and reporting
        - Customer behavior tracking

        ## Important Notes:

        - **Duplicate Prevention**:
          - Duplicate `event_id` values within the same request are rejected (entire
            request fails)
          - Subsequent requests with existing `event_id` values are ignored (idempotent
            behavior)
        - **Rate Limiting**: Maximum 1000 events per request
        - **Time Validation**: Events with timestamps older than 1 hour or more than 5
          minutes in the future will be rejected
        - **Metadata Limits**: Maximum 50 key-value pairs per event, keys max 100 chars,
          values max 500 chars

        ## Example Usage:

        ```json
        {
          "events": [
            {
              "event_id": "api_call_12345",
              "customer_id": "cus_abc123",
              "event_name": "api_request",
              "timestamp": "2024-01-15T10:30:00Z",
              "metadata": {
                "endpoint": "/api/v1/users",
                "method": "GET",
                "tokens_used": "150"
              }
            }
          ]
        }
        ```

        Args:
          events: List of events to be pushed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/events/ingest",
            body=await async_maybe_transform({"events": events}, usage_event_ingest_params.UsageEventIngestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageEventIngestResponse,
        )


class UsageEventsResourceWithRawResponse:
    def __init__(self, usage_events: UsageEventsResource) -> None:
        self._usage_events = usage_events

        self.retrieve = to_raw_response_wrapper(
            usage_events.retrieve,
        )
        self.list = to_raw_response_wrapper(
            usage_events.list,
        )
        self.ingest = to_raw_response_wrapper(
            usage_events.ingest,
        )


class AsyncUsageEventsResourceWithRawResponse:
    def __init__(self, usage_events: AsyncUsageEventsResource) -> None:
        self._usage_events = usage_events

        self.retrieve = async_to_raw_response_wrapper(
            usage_events.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            usage_events.list,
        )
        self.ingest = async_to_raw_response_wrapper(
            usage_events.ingest,
        )


class UsageEventsResourceWithStreamingResponse:
    def __init__(self, usage_events: UsageEventsResource) -> None:
        self._usage_events = usage_events

        self.retrieve = to_streamed_response_wrapper(
            usage_events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            usage_events.list,
        )
        self.ingest = to_streamed_response_wrapper(
            usage_events.ingest,
        )


class AsyncUsageEventsResourceWithStreamingResponse:
    def __init__(self, usage_events: AsyncUsageEventsResource) -> None:
        self._usage_events = usage_events

        self.retrieve = async_to_streamed_response_wrapper(
            usage_events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            usage_events.list,
        )
        self.ingest = async_to_streamed_response_wrapper(
            usage_events.ingest,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["AddMeterToPrice"]


class AddMeterToPrice(BaseModel):
    meter_id: str

    price_per_unit: str
    """The price per unit in lowest denomination.

    Must be greater than zero. Supports up to 5 digits before decimal point and 12
    decimal places.
    """

    description: Optional[str] = None
    """Meter description. Will ignored on Request, but will be shown in response"""

    free_threshold: Optional[int] = None

    measurement_unit: Optional[str] = None
    """Meter measurement unit. Will ignored on Request, but will be shown in response"""

    name: Optional[str] = None
    """Meter name. Will ignored on Request, but will be shown in response"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ledger_entry_list_params import LedgerEntryListParams as LedgerEntryListParams
from .ledger_entry_create_params import LedgerEntryCreateParams as LedgerEntryCreateParams
from .customer_wallet_transaction import CustomerWalletTransaction as CustomerWalletTransaction



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["NewCustomerParam"]


class NewCustomerParam(TypedDict, total=False):
    email: Required[str]
    """Email is required for creating a new customer"""

    name: Optional[str]
    """Optional full name of the customer.

    If provided during session creation, it is persisted and becomes immutable for
    the session. If omitted here, it can be provided later via the confirm API.
    """

    phone_number: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .currency import Currency
from .tax_category import TaxCategory

__all__ = ["AddonUpdateParams"]


class AddonUpdateParams(TypedDict, total=False):
    currency: Optional[Currency]
    """The currency of the Addon"""

    description: Optional[str]
    """Description of the Addon, optional and must be at most 1000 characters."""

    image_id: Optional[str]
    """Addon image id after its uploaded to S3"""

    name: Optional[str]
    """Name of the Addon, optional and must be at most 100 characters."""

    price: Optional[int]
    """Amount of the addon"""

    tax_category: Optional[TaxCategory]
    """Tax category of the Addon."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["CheckoutSessionResponse"]


class CheckoutSessionResponse(BaseModel):
    checkout_url: str
    """Checkout url"""

    session_id: str
    """The ID of the created checkout session"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime

from .._models import BaseModel
from .currency import Currency
from .time_interval import TimeInterval
from .billing_address import BillingAddress
from .subscription_status import SubscriptionStatus
from .addon_cart_response_item import AddonCartResponseItem
from .customer_limited_details import CustomerLimitedDetails

__all__ = ["Subscription", "Meter"]


class Meter(BaseModel):
    currency: Currency

    free_threshold: int

    measurement_unit: str

    meter_id: str

    name: str

    price_per_unit: str

    description: Optional[str] = None


class Subscription(BaseModel):
    addons: List[AddonCartResponseItem]
    """Addons associated with this subscription"""

    billing: BillingAddress
    """Billing address details for payments"""

    cancel_at_next_billing_date: bool
    """Indicates if the subscription will cancel at the next billing date"""

    created_at: datetime
    """Timestamp when the subscription was created"""

    currency: Currency
    """Currency used for the subscription payments"""

    customer: CustomerLimitedDetails
    """Customer details associated with the subscription"""

    metadata: Dict[str, str]
    """Additional custom data associated with the subscription"""

    meters: List[Meter]
    """Meters associated with this subscription (for usage-based billing)"""

    next_billing_date: datetime
    """Timestamp of the next scheduled billing.

    Indicates the end of current billing period
    """

    on_demand: bool
    """Wether the subscription is on-demand or not"""

    payment_frequency_count: int
    """Number of payment frequency intervals"""

    payment_frequency_interval: TimeInterval
    """Time interval for payment frequency (e.g. month, year)"""

    previous_billing_date: datetime
    """Timestamp of the last payment. Indicates the start of current billing period"""

    product_id: str
    """Identifier of the product associated with this subscription"""

    quantity: int
    """Number of units/items included in the subscription"""

    recurring_pre_tax_amount: int
    """
    Amount charged before tax for each recurring payment in smallest currency unit
    (e.g. cents)
    """

    status: SubscriptionStatus
    """Current status of the subscription"""

    subscription_id: str
    """Unique identifier for the subscription"""

    subscription_period_count: int
    """Number of subscription period intervals"""

    subscription_period_interval: TimeInterval
    """Time interval for the subscription period (e.g. month, year)"""

    tax_inclusive: bool
    """Indicates if the recurring_pre_tax_amount is tax inclusive"""

    trial_period_days: int
    """Number of days in the trial period (0 if no trial)"""

    cancelled_at: Optional[datetime] = None
    """Cancelled timestamp if the subscription is cancelled"""

    discount_cycles_remaining: Optional[int] = None
    """Number of remaining discount cycles if discount is applied"""

    discount_id: Optional[str] = None
    """The discount id if discount is applied"""

    expires_at: Optional[datetime] = None
    """Timestamp when the subscription will expire"""

    tax_id: Optional[str] = None
    """Tax identifier provided for this subscription (if applicable)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["RefundCreateParams", "Item"]


class RefundCreateParams(TypedDict, total=False):
    payment_id: Required[str]
    """The unique identifier of the payment to be refunded."""

    items: Optional[Iterable[Item]]
    """Partially Refund an Individual Item"""

    reason: Optional[str]
    """The reason for the refund, if any. Maximum length is 3000 characters. Optional."""


class Item(TypedDict, total=False):
    item_id: Required[str]
    """The id of the item (i.e. `product_id` or `addon_id`)"""

    amount: Optional[int]
    """The amount to refund. if None the whole item is refunded"""

    tax_inclusive: bool
    """Specify if tax is inclusive of the refund. Default true."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast
from datetime import datetime, timezone

import pytest
import standardwebhooks

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    WebhookDetails,
    WebhookRetrieveSecretResponse,
)
from dodopayments.pagination import SyncCursorPagePagination, AsyncCursorPagePagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        webhook = client.webhooks.create(
            url="url",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        webhook = client.webhooks.create(
            url="url",
            description="description",
            disabled=True,
            filter_types=["payment.succeeded"],
            headers={"foo": "string"},
            idempotency_key="idempotency_key",
            metadata={"foo": "string"},
            rate_limit=0,
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.create(
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.create(
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        webhook = client.webhooks.retrieve(
            "webhook_id",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.retrieve(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.retrieve(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: DodoPayments) -> None:
        webhook = client.webhooks.update(
            webhook_id="webhook_id",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: DodoPayments) -> None:
        webhook = client.webhooks.update(
            webhook_id="webhook_id",
            description="description",
            disabled=True,
            filter_types=["payment.succeeded"],
            metadata={"foo": "string"},
            rate_limit=0,
            url="url",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.update(
            webhook_id="webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.update(
            webhook_id="webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.update(
                webhook_id="",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        webhook = client.webhooks.list()
        assert_matches_type(SyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: DodoPayments) -> None:
        webhook = client.webhooks.list(
            iterator="iterator",
            limit=0,
        )
        assert_matches_type(SyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(SyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(SyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: DodoPayments) -> None:
        webhook = client.webhooks.delete(
            "webhook_id",
        )
        assert webhook is None

    @parametrize
    def test_raw_response_delete(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.delete(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert webhook is None

    @parametrize
    def test_streaming_response_delete(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.delete(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert webhook is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_retrieve_secret(self, client: DodoPayments) -> None:
        webhook = client.webhooks.retrieve_secret(
            "webhook_id",
        )
        assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_retrieve_secret(self, client: DodoPayments) -> None:
        response = client.webhooks.with_raw_response.retrieve_secret(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_secret(self, client: DodoPayments) -> None:
        with client.webhooks.with_streaming_response.retrieve_secret(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_secret(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.retrieve_secret(
                "",
            )

    def test_method_unwrap(self, client: DodoPayments) -> None:
        key = b"secret"
        hook = standardwebhooks.Webhook(key)

        data = """{"business_id":"business_id","data":{"amount":"amount","business_id":"business_id","created_at":"2019-12-27T18:11:19.117Z","currency":"currency","dispute_id":"dispute_id","dispute_stage":"pre_dispute","dispute_status":"dispute_opened","payment_id":"payment_id","remarks":"remarks","payload_type":"Dispute"},"timestamp":"2019-12-27T18:11:19.117Z","type":"dispute.accepted"}"""
        msg_id = "1"
        timestamp = datetime.now(tz=timezone.utc)
        sig = hook.sign(msg_id=msg_id, timestamp=timestamp, data=data)
        headers = {
            "webhook-id": msg_id,
            "webhook-timestamp": str(int(timestamp.timestamp())),
            "webhook-signature": sig,
        }

        try:
            _ = client.webhooks.unwrap(data, headers=headers, key=key)
        except standardwebhooks.WebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        bad_headers = [
            {**headers, "webhook-signature": hook.sign(msg_id=msg_id, timestamp=timestamp, data="xxx")},
            {**headers, "webhook-id": "bad"},
            {**headers, "webhook-timestamp": "0"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(standardwebhooks.WebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=key)


class TestAsyncWebhooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.create(
            url="url",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.create(
            url="url",
            description="description",
            disabled=True,
            filter_types=["payment.succeeded"],
            headers={"foo": "string"},
            idempotency_key="idempotency_key",
            metadata={"foo": "string"},
            rate_limit=0,
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.create(
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.create(
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.retrieve(
            "webhook_id",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.retrieve(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.retrieve(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.update(
            webhook_id="webhook_id",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.update(
            webhook_id="webhook_id",
            description="description",
            disabled=True,
            filter_types=["payment.succeeded"],
            metadata={"foo": "string"},
            rate_limit=0,
            url="url",
        )
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.update(
            webhook_id="webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookDetails, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.update(
            webhook_id="webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookDetails, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.update(
                webhook_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.list()
        assert_matches_type(AsyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.list(
            iterator="iterator",
            limit=0,
        )
        assert_matches_type(AsyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(AsyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(AsyncCursorPagePagination[WebhookDetails], webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.delete(
            "webhook_id",
        )
        assert webhook is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.delete(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert webhook is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.delete(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert webhook is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_retrieve_secret(self, async_client: AsyncDodoPayments) -> None:
        webhook = await async_client.webhooks.retrieve_secret(
            "webhook_id",
        )
        assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_secret(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.webhooks.with_raw_response.retrieve_secret(
            "webhook_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_secret(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.webhooks.with_streaming_response.retrieve_secret(
            "webhook_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookRetrieveSecretResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_secret(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.retrieve_secret(
                "",
            )

    def test_method_unwrap(self, client: DodoPayments) -> None:
        key = b"secret"
        hook = standardwebhooks.Webhook(key)

        data = """{"business_id":"business_id","data":{"amount":"amount","business_id":"business_id","created_at":"2019-12-27T18:11:19.117Z","currency":"currency","dispute_id":"dispute_id","dispute_stage":"pre_dispute","dispute_status":"dispute_opened","payment_id":"payment_id","remarks":"remarks","payload_type":"Dispute"},"timestamp":"2019-12-27T18:11:19.117Z","type":"dispute.accepted"}"""
        msg_id = "1"
        timestamp = datetime.now(tz=timezone.utc)
        sig = hook.sign(msg_id=msg_id, timestamp=timestamp, data=data)
        headers = {
            "webhook-id": msg_id,
            "webhook-timestamp": str(int(timestamp.timestamp())),
            "webhook-signature": sig,
        }

        try:
            _ = client.webhooks.unwrap(data, headers=headers, key=key)
        except standardwebhooks.WebhookVerificationError as e:
            raise AssertionError("Failed to unwrap valid webhook") from e

        bad_headers = [
            {**headers, "webhook-signature": hook.sign(msg_id=msg_id, timestamp=timestamp, data="xxx")},
            {**headers, "webhook-id": "bad"},
            {**headers, "webhook-timestamp": "0"},
        ]
        for bad_header in bad_headers:
            with pytest.raises(standardwebhooks.WebhookVerificationError):
                _ = client.webhooks.unwrap(data, headers=bad_header, key=key)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .wallets import (
    WalletsResource,
    AsyncWalletsResource,
    WalletsResourceWithRawResponse,
    AsyncWalletsResourceWithRawResponse,
    WalletsResourceWithStreamingResponse,
    AsyncWalletsResourceWithStreamingResponse,
)
from .ledger_entries import (
    LedgerEntriesResource,
    AsyncLedgerEntriesResource,
    LedgerEntriesResourceWithRawResponse,
    AsyncLedgerEntriesResourceWithRawResponse,
    LedgerEntriesResourceWithStreamingResponse,
    AsyncLedgerEntriesResourceWithStreamingResponse,
)

__all__ = [
    "LedgerEntriesResource",
    "AsyncLedgerEntriesResource",
    "LedgerEntriesResourceWithRawResponse",
    "AsyncLedgerEntriesResourceWithRawResponse",
    "LedgerEntriesResourceWithStreamingResponse",
    "AsyncLedgerEntriesResourceWithStreamingResponse",
    "WalletsResource",
    "AsyncWalletsResource",
    "WalletsResourceWithRawResponse",
    "AsyncWalletsResourceWithRawResponse",
    "WalletsResourceWithStreamingResponse",
    "AsyncWalletsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from dodopayments import DodoPayments, AsyncDodoPayments
from dodopayments.types import (
    Product,
    ProductListResponse,
    ProductUpdateFilesResponse,
)
from dodopayments.pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestProducts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: DodoPayments) -> None:
        product = client.products.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: DodoPayments) -> None:
        product = client.products.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
                "pay_what_you_want": True,
                "suggested_price": 0,
                "tax_inclusive": True,
            },
            tax_category="digital_products",
            addons=["string"],
            brand_id="brand_id",
            description="description",
            digital_product_delivery={
                "external_url": "external_url",
                "instructions": "instructions",
            },
            license_key_activation_message="license_key_activation_message",
            license_key_activations_limit=0,
            license_key_duration={
                "count": 0,
                "interval": "Day",
            },
            license_key_enabled=True,
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: DodoPayments) -> None:
        product = client.products.retrieve(
            "id",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: DodoPayments) -> None:
        product = client.products.update(
            id="id",
        )
        assert product is None

    @parametrize
    def test_method_update_with_all_params(self, client: DodoPayments) -> None:
        product = client.products.update(
            id="id",
            addons=["string"],
            brand_id="brand_id",
            description="description",
            digital_product_delivery={
                "external_url": "external_url",
                "files": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "instructions": "instructions",
            },
            image_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            license_key_activation_message="license_key_activation_message",
            license_key_activations_limit=0,
            license_key_duration={
                "count": 0,
                "interval": "Day",
            },
            license_key_enabled=True,
            metadata={"foo": "string"},
            name="name",
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
                "pay_what_you_want": True,
                "suggested_price": 0,
                "tax_inclusive": True,
            },
            tax_category="digital_products",
        )
        assert product is None

    @parametrize
    def test_raw_response_update(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert product is None

    @parametrize
    def test_streaming_response_update(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: DodoPayments) -> None:
        product = client.products.list()
        assert_matches_type(SyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: DodoPayments) -> None:
        product = client.products.list(
            archived=True,
            brand_id="brand_id",
            page_number=0,
            page_size=0,
            recurring=True,
        )
        assert_matches_type(SyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(SyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(SyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: DodoPayments) -> None:
        product = client.products.archive(
            "id",
        )
        assert product is None

    @parametrize
    def test_raw_response_archive(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.archive(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert product is None

    @parametrize
    def test_streaming_response_archive(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.archive(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_archive(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.archive(
                "",
            )

    @parametrize
    def test_method_unarchive(self, client: DodoPayments) -> None:
        product = client.products.unarchive(
            "id",
        )
        assert product is None

    @parametrize
    def test_raw_response_unarchive(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.unarchive(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert product is None

    @parametrize
    def test_streaming_response_unarchive(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.unarchive(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_unarchive(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.unarchive(
                "",
            )

    @parametrize
    def test_method_update_files(self, client: DodoPayments) -> None:
        product = client.products.update_files(
            id="id",
            file_name="file_name",
        )
        assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

    @parametrize
    def test_raw_response_update_files(self, client: DodoPayments) -> None:
        response = client.products.with_raw_response.update_files(
            id="id",
            file_name="file_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = response.parse()
        assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

    @parametrize
    def test_streaming_response_update_files(self, client: DodoPayments) -> None:
        with client.products.with_streaming_response.update_files(
            id="id",
            file_name="file_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = response.parse()
            assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_files(self, client: DodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.products.with_raw_response.update_files(
                id="",
                file_name="file_name",
            )


class TestAsyncProducts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
                "pay_what_you_want": True,
                "suggested_price": 0,
                "tax_inclusive": True,
            },
            tax_category="digital_products",
            addons=["string"],
            brand_id="brand_id",
            description="description",
            digital_product_delivery={
                "external_url": "external_url",
                "instructions": "instructions",
            },
            license_key_activation_message="license_key_activation_message",
            license_key_activations_limit=0,
            license_key_duration={
                "count": 0,
                "interval": "Day",
            },
            license_key_enabled=True,
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.create(
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
            },
            tax_category="digital_products",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.retrieve(
            "id",
        )
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(Product, product, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(Product, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.update(
            id="id",
        )
        assert product is None

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.update(
            id="id",
            addons=["string"],
            brand_id="brand_id",
            description="description",
            digital_product_delivery={
                "external_url": "external_url",
                "files": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "instructions": "instructions",
            },
            image_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            license_key_activation_message="license_key_activation_message",
            license_key_activations_limit=0,
            license_key_duration={
                "count": 0,
                "interval": "Day",
            },
            license_key_enabled=True,
            metadata={"foo": "string"},
            name="name",
            price={
                "currency": "AED",
                "discount": 0,
                "price": 0,
                "purchasing_power_parity": True,
                "type": "one_time_price",
                "pay_what_you_want": True,
                "suggested_price": 0,
                "tax_inclusive": True,
            },
            tax_category="digital_products",
        )
        assert product is None

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert product is None

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.list()
        assert_matches_type(AsyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.list(
            archived=True,
            brand_id="brand_id",
            page_number=0,
            page_size=0,
            recurring=True,
        )
        assert_matches_type(AsyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(AsyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(AsyncDefaultPageNumberPagination[ProductListResponse], product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.archive(
            "id",
        )
        assert product is None

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.archive(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert product is None

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.archive(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_archive(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.archive(
                "",
            )

    @parametrize
    async def test_method_unarchive(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.unarchive(
            "id",
        )
        assert product is None

    @parametrize
    async def test_raw_response_unarchive(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.unarchive(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert product is None

    @parametrize
    async def test_streaming_response_unarchive(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.unarchive(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert product is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_unarchive(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.unarchive(
                "",
            )

    @parametrize
    async def test_method_update_files(self, async_client: AsyncDodoPayments) -> None:
        product = await async_client.products.update_files(
            id="id",
            file_name="file_name",
        )
        assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

    @parametrize
    async def test_raw_response_update_files(self, async_client: AsyncDodoPayments) -> None:
        response = await async_client.products.with_raw_response.update_files(
            id="id",
            file_name="file_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        product = await response.parse()
        assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

    @parametrize
    async def test_streaming_response_update_files(self, async_client: AsyncDodoPayments) -> None:
        async with async_client.products.with_streaming_response.update_files(
            id="id",
            file_name="file_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            product = await response.parse()
            assert_matches_type(ProductUpdateFilesResponse, product, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_files(self, async_client: AsyncDodoPayments) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.products.with_raw_response.update_files(
                id="",
                file_name="file_name",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.webhooks import header_update_params
from ...types.webhooks.header_retrieve_response import HeaderRetrieveResponse

__all__ = ["HeadersResource", "AsyncHeadersResource"]


class HeadersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HeadersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return HeadersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HeadersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return HeadersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HeaderRetrieveResponse:
        """
        Get a webhook by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._get(
            f"/webhooks/{webhook_id}/headers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HeaderRetrieveResponse,
        )

    def update(
        self,
        webhook_id: str,
        *,
        headers: Dict[str, str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Patch a webhook by id

        Args:
          headers: Object of header-value pair to update or add

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/webhooks/{webhook_id}/headers",
            body=maybe_transform({"headers": headers}, header_update_params.HeaderUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncHeadersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHeadersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHeadersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHeadersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncHeadersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HeaderRetrieveResponse:
        """
        Get a webhook by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._get(
            f"/webhooks/{webhook_id}/headers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HeaderRetrieveResponse,
        )

    async def update(
        self,
        webhook_id: str,
        *,
        headers: Dict[str, str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Patch a webhook by id

        Args:
          headers: Object of header-value pair to update or add

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/webhooks/{webhook_id}/headers",
            body=await async_maybe_transform({"headers": headers}, header_update_params.HeaderUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class HeadersResourceWithRawResponse:
    def __init__(self, headers: HeadersResource) -> None:
        self._headers = headers

        self.retrieve = to_raw_response_wrapper(
            headers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            headers.update,
        )


class AsyncHeadersResourceWithRawResponse:
    def __init__(self, headers: AsyncHeadersResource) -> None:
        self._headers = headers

        self.retrieve = async_to_raw_response_wrapper(
            headers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            headers.update,
        )


class HeadersResourceWithStreamingResponse:
    def __init__(self, headers: HeadersResource) -> None:
        self._headers = headers

        self.retrieve = to_streamed_response_wrapper(
            headers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            headers.update,
        )


class AsyncHeadersResourceWithStreamingResponse:
    def __init__(self, headers: AsyncHeadersResource) -> None:
        self._headers = headers

        self.retrieve = async_to_streamed_response_wrapper(
            headers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            headers.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["OneTimeProductCartItemParam"]


class OneTimeProductCartItemParam(TypedDict, total=False):
    product_id: Required[str]

    quantity: Required[int]

    amount: Optional[int]
    """Amount the customer pays if pay_what_you_want is enabled.

    If disabled then amount will be ignored Represented in the lowest denomination
    of the currency (e.g., cents for USD). For example, to charge $1.00, pass `100`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .event_input_param import EventInputParam

__all__ = ["UsageEventIngestParams"]


class UsageEventIngestParams(TypedDict, total=False):
    events: Required[Iterable[EventInputParam]]
    """List of events to be pushed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import license_key_instance_list_params, license_key_instance_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.license_key_instance import LicenseKeyInstance

__all__ = ["LicenseKeyInstancesResource", "AsyncLicenseKeyInstancesResource"]


class LicenseKeyInstancesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LicenseKeyInstancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return LicenseKeyInstancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LicenseKeyInstancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return LicenseKeyInstancesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/license_key_instances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    def update(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/license_key_instances/{id}",
            body=maybe_transform({"name": name}, license_key_instance_update_params.LicenseKeyInstanceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    def list(
        self,
        *,
        license_key_id: Optional[str] | Omit = omit,
        page_number: Optional[int] | Omit = omit,
        page_size: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[LicenseKeyInstance]:
        """
        Args:
          license_key_id: Filter by license key ID

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/license_key_instances",
            page=SyncDefaultPageNumberPagination[LicenseKeyInstance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "license_key_id": license_key_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    license_key_instance_list_params.LicenseKeyInstanceListParams,
                ),
            ),
            model=LicenseKeyInstance,
        )


class AsyncLicenseKeyInstancesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLicenseKeyInstancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLicenseKeyInstancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLicenseKeyInstancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncLicenseKeyInstancesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/license_key_instances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    async def update(
        self,
        id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LicenseKeyInstance:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/license_key_instances/{id}",
            body=await async_maybe_transform(
                {"name": name}, license_key_instance_update_params.LicenseKeyInstanceUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LicenseKeyInstance,
        )

    def list(
        self,
        *,
        license_key_id: Optional[str] | Omit = omit,
        page_number: Optional[int] | Omit = omit,
        page_size: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LicenseKeyInstance, AsyncDefaultPageNumberPagination[LicenseKeyInstance]]:
        """
        Args:
          license_key_id: Filter by license key ID

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/license_key_instances",
            page=AsyncDefaultPageNumberPagination[LicenseKeyInstance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "license_key_id": license_key_id,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    license_key_instance_list_params.LicenseKeyInstanceListParams,
                ),
            ),
            model=LicenseKeyInstance,
        )


class LicenseKeyInstancesResourceWithRawResponse:
    def __init__(self, license_key_instances: LicenseKeyInstancesResource) -> None:
        self._license_key_instances = license_key_instances

        self.retrieve = to_raw_response_wrapper(
            license_key_instances.retrieve,
        )
        self.update = to_raw_response_wrapper(
            license_key_instances.update,
        )
        self.list = to_raw_response_wrapper(
            license_key_instances.list,
        )


class AsyncLicenseKeyInstancesResourceWithRawResponse:
    def __init__(self, license_key_instances: AsyncLicenseKeyInstancesResource) -> None:
        self._license_key_instances = license_key_instances

        self.retrieve = async_to_raw_response_wrapper(
            license_key_instances.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            license_key_instances.update,
        )
        self.list = async_to_raw_response_wrapper(
            license_key_instances.list,
        )


class LicenseKeyInstancesResourceWithStreamingResponse:
    def __init__(self, license_key_instances: LicenseKeyInstancesResource) -> None:
        self._license_key_instances = license_key_instances

        self.retrieve = to_streamed_response_wrapper(
            license_key_instances.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            license_key_instances.update,
        )
        self.list = to_streamed_response_wrapper(
            license_key_instances.list,
        )


class AsyncLicenseKeyInstancesResourceWithStreamingResponse:
    def __init__(self, license_key_instances: AsyncLicenseKeyInstancesResource) -> None:
        self._license_key_instances = license_key_instances

        self.retrieve = async_to_streamed_response_wrapper(
            license_key_instances.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            license_key_instances.update,
        )
        self.list = async_to_streamed_response_wrapper(
            license_key_instances.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import meter_list_params, meter_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultPageNumberPagination, AsyncDefaultPageNumberPagination
from ..types.meter import Meter
from .._base_client import AsyncPaginator, make_request_options
from ..types.meter_filter_param import MeterFilterParam
from ..types.meter_aggregation_param import MeterAggregationParam

__all__ = ["MetersResource", "AsyncMetersResource"]


class MetersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MetersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return MetersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MetersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return MetersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        aggregation: MeterAggregationParam,
        event_name: str,
        measurement_unit: str,
        name: str,
        description: Optional[str] | Omit = omit,
        filter: Optional[MeterFilterParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Meter:
        """
        Args:
          aggregation: Aggregation configuration for the meter

          event_name: Event name to track

          measurement_unit: measurement unit

          name: Name of the meter

          description: Optional description of the meter

          filter: Optional filter to apply to the meter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/meters",
            body=maybe_transform(
                {
                    "aggregation": aggregation,
                    "event_name": event_name,
                    "measurement_unit": measurement_unit,
                    "name": name,
                    "description": description,
                    "filter": filter,
                },
                meter_create_params.MeterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Meter,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Meter:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/meters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Meter,
        )

    def list(
        self,
        *,
        archived: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPageNumberPagination[Meter]:
        """
        Args:
          archived: List archived meters

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/meters",
            page=SyncDefaultPageNumberPagination[Meter],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    meter_list_params.MeterListParams,
                ),
            ),
            model=Meter,
        )

    def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/meters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/meters/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncMetersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMetersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMetersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMetersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/dodopayments/dodopayments-python#with_streaming_response
        """
        return AsyncMetersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        aggregation: MeterAggregationParam,
        event_name: str,
        measurement_unit: str,
        name: str,
        description: Optional[str] | Omit = omit,
        filter: Optional[MeterFilterParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Meter:
        """
        Args:
          aggregation: Aggregation configuration for the meter

          event_name: Event name to track

          measurement_unit: measurement unit

          name: Name of the meter

          description: Optional description of the meter

          filter: Optional filter to apply to the meter

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/meters",
            body=await async_maybe_transform(
                {
                    "aggregation": aggregation,
                    "event_name": event_name,
                    "measurement_unit": measurement_unit,
                    "name": name,
                    "description": description,
                    "filter": filter,
                },
                meter_create_params.MeterCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Meter,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Meter:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/meters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Meter,
        )

    def list(
        self,
        *,
        archived: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Meter, AsyncDefaultPageNumberPagination[Meter]]:
        """
        Args:
          archived: List archived meters

          page_number: Page number default is 0

          page_size: Page size default is 10 max is 100

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/meters",
            page=AsyncDefaultPageNumberPagination[Meter],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "archived": archived,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    meter_list_params.MeterListParams,
                ),
            ),
            model=Meter,
        )

    async def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/meters/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/meters/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class MetersResourceWithRawResponse:
    def __init__(self, meters: MetersResource) -> None:
        self._meters = meters

        self.create = to_raw_response_wrapper(
            meters.create,
        )
        self.retrieve = to_raw_response_wrapper(
            meters.retrieve,
        )
        self.list = to_raw_response_wrapper(
            meters.list,
        )
        self.archive = to_raw_response_wrapper(
            meters.archive,
        )
        self.unarchive = to_raw_response_wrapper(
            meters.unarchive,
        )


class AsyncMetersResourceWithRawResponse:
    def __init__(self, meters: AsyncMetersResource) -> None:
        self._meters = meters

        self.create = async_to_raw_response_wrapper(
            meters.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            meters.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            meters.list,
        )
        self.archive = async_to_raw_response_wrapper(
            meters.archive,
        )
        self.unarchive = async_to_raw_response_wrapper(
            meters.unarchive,
        )


class MetersResourceWithStreamingResponse:
    def __init__(self, meters: MetersResource) -> None:
        self._meters = meters

        self.create = to_streamed_response_wrapper(
            meters.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            meters.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            meters.list,
        )
        self.archive = to_streamed_response_wrapper(
            meters.archive,
        )
        self.unarchive = to_streamed_response_wrapper(
            meters.unarchive,
        )


class AsyncMetersResourceWithStreamingResponse:
    def __init__(self, meters: AsyncMetersResource) -> None:
        self._meters = meters

        self.create = async_to_streamed_response_wrapper(
            meters.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            meters.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            meters.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            meters.archive,
        )
        self.unarchive = async_to_streamed_response_wrapper(
            meters.unarchive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel
from .meter_filter import MeterFilter
from .meter_aggregation import MeterAggregation

__all__ = ["Meter"]


class Meter(BaseModel):
    id: str

    aggregation: MeterAggregation

    business_id: str

    created_at: datetime

    event_name: str

    measurement_unit: str

    name: str

    updated_at: datetime

    description: Optional[str] = None

    filter: Optional[MeterFilter] = None
    """
    A filter structure that combines multiple conditions with logical conjunctions
    (AND/OR).

    Supports up to 3 levels of nesting to create complex filter expressions. Each
    filter has a conjunction (and/or) and clauses that can be either direct
    conditions or nested filters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UsageEventListParams"]


class UsageEventListParams(TypedDict, total=False):
    customer_id: str
    """Filter events by customer ID"""

    end: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter events created before this timestamp"""

    event_name: str
    """Filter events by event name.

    If both event_name and meter_id are provided, they must match the meter's
    configured event_name
    """

    meter_id: str
    """Filter events by meter ID.

    When provided, only events that match the meter's event_name and filter criteria
    will be returned
    """

    page_number: int
    """Page number (0-based, default: 0)"""

    page_size: int
    """Number of events to return per page (default: 10)"""

    start: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter events created after this timestamp"""
